@@ -89,6 +89,7 @@ const char *snd_config_topdir(void);
 int snd_config_top(snd_config_t **config);
 
 int snd_config_load(snd_config_t *config, snd_input_t *in);
+int snd_config_load_string(snd_config_t **config, const char *s, size_t size);
 int snd_config_load_override(snd_config_t *config, snd_input_t *in);
 int snd_config_save(snd_config_t *config, snd_output_t *out);
 int snd_config_update(void);
@@ -108,11 +109,18 @@ int snd_config_search_definition(snd_config_t *config,
 				 const char *base, const char *key,
 				 snd_config_t **result);
 
+typedef int (*snd_config_expand_fcn_t)(snd_config_t **dst, const char *s, void *private_data);
+
+int snd_config_expand_custom(snd_config_t *config, snd_config_t *root,
+			     snd_config_expand_fcn_t fcn, void *private_data,
+			     snd_config_t **result);
 int snd_config_expand(snd_config_t *config, snd_config_t *root,
 		      const char *args, snd_config_t *private_data,
 		      snd_config_t **result);
 int snd_config_evaluate(snd_config_t *config, snd_config_t *root,
 			snd_config_t *private_data, snd_config_t **result);
+int snd_config_evaluate_string(snd_config_t **dst, const char *s,
+			       snd_config_expand_fcn_t fcn, void *private_data);
 
 int snd_config_add(snd_config_t *config, snd_config_t *child);
 int snd_config_add_before(snd_config_t *before, snd_config_t *child);
@@ -121,6 +129,7 @@ int snd_config_remove(snd_config_t *config);
 int snd_config_delete(snd_config_t *config);
 int snd_config_delete_compound_members(const snd_config_t *config);
 int snd_config_copy(snd_config_t **dst, snd_config_t *src);
+int snd_config_merge(snd_config_t *dst, snd_config_t *src, int override);
 
 int snd_config_make(snd_config_t **config, const char *key,
 		    snd_config_type_t type);
@@ -130,6 +139,8 @@ int snd_config_make_real(snd_config_t **config, const char *key);
 int snd_config_make_string(snd_config_t **config, const char *key);
 int snd_config_make_pointer(snd_config_t **config, const char *key);
 int snd_config_make_compound(snd_config_t **config, const char *key, int join);
+int snd_config_make_path(snd_config_t **config, snd_config_t *root, const char *key,
+			 int join, int override);
 
 int snd_config_imake_integer(snd_config_t **config, const char *key, const long value);
 int snd_config_imake_integer64(snd_config_t **config, const char *key, const long long value);
@@ -139,6 +150,8 @@ int snd_config_imake_safe_string(snd_config_t **config, const char *key, const c
 int snd_config_imake_pointer(snd_config_t **config, const char *key, const void *ptr);
 
 snd_config_type_t snd_config_get_type(const snd_config_t *config);
+int snd_config_is_array(const snd_config_t *config);
+int snd_config_is_empty(const snd_config_t *config);
 
 int snd_config_set_id(snd_config_t *config, const char *id);
 int snd_config_set_integer(snd_config_t *config, long value);
@@ -186,6 +199,7 @@ snd_config_t *snd_config_iterator_entry(const snd_config_iterator_t iterator);
 
 int snd_config_get_bool_ascii(const char *ascii);
 int snd_config_get_bool(const snd_config_t *conf);
+int snd_config_get_card(const snd_config_t *conf);
 int snd_config_get_ctl_iface_ascii(const char *ascii);
 int snd_config_get_ctl_iface(const snd_config_t *conf);
 
