@@ -232,7 +232,10 @@ size_t page_align(size_t size);
 size_t page_size(void);
 size_t page_ptr(size_t object_offset, size_t object_size, size_t *offset, size_t *mmap_offset);
 
-int safe_strtol(const char *str, long *val);
+int safe_strtoll_base(const char *str, long long *val, int base);
+static inline int safe_strtoll(const char *str, long long *val) { return safe_strtoll_base(str, val, 0); }
+int safe_strtol_base(const char *str, long *val, int base);
+static inline int safe_strtol(const char *str, long *val) { return safe_strtol_base(str, val, 0); }
 
 int snd_send_fd(int sock, void *data, size_t len, int fd);
 int snd_receive_fd(int sock, void *data, size_t len, int *fd);
@@ -374,4 +377,21 @@ int _snd_config_load_with_include(snd_config_t *config, snd_input_t *in,
 void *INTERNAL(snd_dlopen)(const char *name, int mode, char *errbuf, size_t errbuflen);
 #endif
 
+#ifdef BUILD_UCM
+
+const char *uc_mgr_alibcfg_by_device(snd_config_t **config, const char *name);
+
+static inline int _snd_is_ucm_device(const char *name)
+{
+	return name && name[0] == '_' && name[1] == 'u' && name[2] == 'c' && name[3] == 'm';
+}
+
+#else
+
+static inline const char *uc_mgr_alibcfg_by_device(snd_config_t **config, const char *name) { return NULL; }
+static inline int _snd_is_ucm_device(const char *name) { return 0; }
+
+
+#endif
+
 #endif
