@@ -37,6 +37,7 @@
 #include <stdarg.h>
 #include <pthread.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <limits.h>
 
 /*
@@ -605,8 +606,10 @@ static int execute_cfgsave(snd_use_case_mgr_t *uc_mgr, const char *filename)
 		uc_error("unable to open file '%s': %s", file, snd_strerror(err));
 		goto _err;
 	}
-	if (!config || snd_config_is_empty(config))
+	if (!config || snd_config_is_empty(config)) {
+		snd_output_close(out);
 		goto _err;
+	}
 	if (with_root) {
 		snd_output_printf(out, "%s ", root);
 		err = _snd_config_save_node_value(config, out, 0);
@@ -1380,6 +1383,11 @@ int snd_use_case_mgr_open(snd_use_case_mgr_t **uc_mgr,
 	INIT_LIST_HEAD(&mgr->variable_list);
 	pthread_mutex_init(&mgr->mutex, NULL);
 
+	if (card_name && *card_name == '-') {
+		card_name++;
+		mgr->suppress_nodev_errors = 1;
+	}
+
 	err = uc_mgr_card_open(mgr);
 	if (err < 0) {
 		uc_mgr_free(mgr);
@@ -1399,6 +1407,8 @@ int snd_use_case_mgr_open(snd_use_case_mgr_t **uc_mgr,
 	/* get info on use_cases and verify against card */
 	err = import_master_config(mgr);
 	if (err < 0) {
+		if (err == -ENXIO && mgr->suppress_nodev_errors)
+			goto _err;
 		uc_error("error: failed to import %s use case configuration %d",
 			 card_name, err);
 		goto _err;
@@ -2138,6 +2148,25 @@ static int get_alibcfg(snd_use_case_mgr_t *uc_mgr, char **str)
 	return 0;
 }
 
+/**
+ * \brief Get device prefix for private alsa-lib configuration
+ * \param uc_mgr Use case manager
+ * \param str Returned value string
+ * \return Zero on success (value is filled), otherwise a negative error code
+ */
+static int get_alibpref(snd_use_case_mgr_t *uc_mgr, char **str)
+{
+	const size_t l = 10;
+	char *s;
+
+	s = malloc(l);
+	if (s == NULL)
+		return -ENOMEM;
+	snprintf(s, l, "_ucm%04X.", uc_mgr->ucm_card_number);
+	*str = s;
+	return 0;
+}
+
 /**
  * \brief Get current - string
  * \param uc_mgr Use case manager
@@ -2193,6 +2222,8 @@ int snd_use_case_get(snd_use_case_mgr_t *uc_mgr,
 
 	} else if (strcmp(identifier, "_alibcfg") == 0) {
 		err = get_alibcfg(uc_mgr, (char **)value);
+	} else if (strcmp(identifier, "_alibpref") == 0) {
+		err = get_alibpref(uc_mgr, (char **)value);
 	} else if (identifier[0] == '_') {
 		err = -ENOENT;
 	} else {
