@@ -239,7 +239,8 @@ static int error_node(snd_use_case_mgr_t *uc_mgr, snd_config_t *cfg)
 		uc_error("error: failed to get Error string");
 		return err;
 	}
-	uc_error("%s", s);
+	if (!uc_mgr->suppress_nodev_errors)
+		uc_error("%s", s);
 	free(s);
 	return -ENXIO;
 }
@@ -1469,7 +1470,7 @@ static int parse_modifier_name(snd_use_case_mgr_t *uc_mgr,
 			     void *data1,
 			     void *data2 ATTRIBUTE_UNUSED)
 {
-	return parse_compound(uc_mgr, cfg, parse_modifier, data1, data2);
+	return parse_compound_check_legacy(uc_mgr, cfg, parse_modifier, data1);
 }
 
 static int verb_dev_list_add(struct use_case_verb *verb,
@@ -1779,7 +1780,7 @@ static int parse_verb_file(snd_use_case_mgr_t *uc_mgr,
 			err = parse_libconfig(uc_mgr, n);
 			if (err < 0) {
 				uc_error("error: failed to parse LibConfig");
-				return err;
+				goto _err;
 			}
 			continue;
 		}
@@ -2518,7 +2519,11 @@ int uc_mgr_scan_master_configs(const char **_list[])
 
 		snprintf(fn, sizeof(fn), "%s.conf", d_name);
 		ucm_filename(filename, sizeof(filename), 2, d_name, fn);
+#ifdef HAVE_EACCESS
 		if (eaccess(filename, R_OK))
+#else
+		if (access(filename, R_OK))
+#endif
 			continue;
 
 		err = uc_mgr_config_load(2, filename, &cfg);
