@@ -91,24 +91,30 @@ static void config_dump(snd_config_t *cfg)
 }
 #endif
 
-static int find_position_node(snd_config_t **res, snd_config_t *dst,
+static int find_position_node(snd_use_case_mgr_t *uc_mgr,
+			      snd_config_t **res, snd_config_t *dst,
 			      const char *id, snd_config_t *pos)
 {
 	const char *s;
+	char *s1;
 	int err;
 
 	err = get_string(pos, id, &s);
 	if (err < 0 && err != -ENOENT)
 		return err;
 	if (err == 0) {
-		err = snd_config_search(dst, s, res);
+		err = uc_mgr_get_substituted_value(uc_mgr, &s1, s);
+		if (err < 0)
+			return err;
+		err = snd_config_search(dst, s1, res);
+		free(s1);
 		if (err < 0 && err != -ENOENT)
 			return err;
 	}
 	return 0;
 }
 
-static int merge_it(snd_config_t *dst, snd_config_t *n)
+static int merge_it(snd_config_t *dst, snd_config_t *n, snd_config_t **_dn)
 {
 	snd_config_t *dn;
 	const char *id;
@@ -123,10 +129,12 @@ static int merge_it(snd_config_t *dst, snd_config_t *n)
 	err = snd_config_merge(dn, n, 0); /* merge / append mode */
 	if (err < 0)
 		snd_config_delete(n);
+	else
+		*_dn = dn;
 	return err;
 }
 
-static int compound_merge(const char *id,
+static int compound_merge(snd_use_case_mgr_t *uc_mgr, const char *id,
 			  snd_config_t *dst, snd_config_t *src,
 			  snd_config_t *before, snd_config_t *after)
 {
@@ -141,12 +149,12 @@ static int compound_merge(const char *id,
 	}
 
 	if (before) {
-		err = find_position_node(&_before, dst, id, before);
+		err = find_position_node(uc_mgr, &_before, dst, id, before);
 		if (err < 0)
 			return err;
 	}
 	if (after) {
-		err = find_position_node(&_after, dst, id, after);
+		err = find_position_node(uc_mgr, &_after, dst, id, after);
 		if (err < 0)
 			return err;
 	}
@@ -198,7 +206,7 @@ static int compound_merge(const char *id,
 		if (_before) {
 			err = snd_config_add_before(_before, n);
 			if (err == -EEXIST)
-				err = merge_it(dst, n);
+				err = merge_it(dst, n, &n);
 			if (err < 0)
 				return err;
 			_before = NULL;
@@ -206,7 +214,7 @@ static int compound_merge(const char *id,
 		} else if (_after) {
 			err = snd_config_add_after(_after, n);
 			if (err == -EEXIST)
-				err = merge_it(dst, n);
+				err = merge_it(dst, n, &n);
 			if (err < 0)
 				return err;
 			_after = n;
@@ -257,7 +265,7 @@ __add:
 			err = snd_config_search(parent, id, &parent2);
 			if (err == -ENOENT)
 				goto __add;
-			err = compound_merge(id, parent2, n, before, after);
+			err = compound_merge(uc_mgr, id, parent2, n, before, after);
 			if (err < 0) {
 				snd_config_delete(n);
 				return err;
