@@ -49,6 +49,16 @@ void uc_mgr_stdout(const char *fmt,...)
 	va_end(va);
 }
 
+const char *uc_mgr_sysfs_root(void)
+{
+	const char *e = getenv("SYSFS_PATH");
+	if (e == NULL)
+		return "/sys";
+	if (*e == '\0')
+		uc_error("no sysfs root!");
+	return e;
+}
+
 struct ctl_list *uc_mgr_get_master_ctl(snd_use_case_mgr_t *uc_mgr)
 {
 	struct list_head *pos;
@@ -67,13 +77,25 @@ struct ctl_list *uc_mgr_get_master_ctl(snd_use_case_mgr_t *uc_mgr)
 	return ctl_list;
 }
 
+struct ctl_list *uc_mgr_get_ctl_by_card(snd_use_case_mgr_t *uc_mgr, int card)
+{
+	struct ctl_list *ctl_list;
+	char cname[32];
+	int err;
+
+	sprintf(cname, "hw:%d", card);
+	err = uc_mgr_open_ctl(uc_mgr, &ctl_list, cname, 1);
+	if (err < 0)
+		return NULL;
+	return ctl_list;
+}
+
 struct ctl_list *uc_mgr_get_ctl_by_name(snd_use_case_mgr_t *uc_mgr, const char *name, int idx)
 {
 	struct list_head *pos;
-	struct ctl_list *ctl_list = NULL;
+	struct ctl_list *ctl_list;
 	const char *s;
-	char cname[32];
-	int idx2, card, err;
+	int idx2, card;
 
 	idx2 = idx;
 	list_for_each(pos, &uc_mgr->ctl_list) {
@@ -94,9 +116,8 @@ struct ctl_list *uc_mgr_get_ctl_by_name(snd_use_case_mgr_t *uc_mgr, const char *
 		return NULL;
 
 	while (card >= 0) {
-		sprintf(cname, "hw:%d", card);
-		err = uc_mgr_open_ctl(uc_mgr, &ctl_list, cname, 1);
-		if (err < 0)
+		ctl_list = uc_mgr_get_ctl_by_card(uc_mgr, card);
+		if (ctl_list == NULL)
 			continue;	/* really? */
 		s = snd_ctl_card_info_get_name(ctl_list->ctl_info);
 		if (s && strcmp(s, name) == 0) {
@@ -245,16 +266,21 @@ int uc_mgr_open_ctl(snd_use_case_mgr_t *uc_mgr,
 	struct ctl_dev *ctl_dev;
 	snd_ctl_card_info_t *info;
 	const char *id;
-	int err, card;
+	int err, card, ucm_group, ucm_offset;
 
 	snd_ctl_card_info_alloca(&info);
 
+	ucm_group = _snd_is_ucm_device(device);
+	ucm_offset = ucm_group ? 8 : 0;
+
 	/* cache lookup */
 	list_for_each(pos1, &uc_mgr->ctl_list) {
 		ctl_list = list_entry(pos1, struct ctl_list, list);
+		if (ctl_list->ucm_group != ucm_group)
+			continue;
 		list_for_each(pos2, &ctl_list->dev_list) {
 			ctl_dev = list_entry(pos2, struct ctl_dev, list);
-			if (strcmp(ctl_dev->device, device) == 0) {
+			if (strcmp(ctl_dev->device, device + ucm_offset) == 0) {
 				*ctll = ctl_list;
 				if (!slave)
 					ctl_list->slave = 0;
@@ -280,22 +306,26 @@ int uc_mgr_open_ctl(snd_use_case_mgr_t *uc_mgr,
 	/* insert to cache, if just name differs */
 	list_for_each(pos1, &uc_mgr->ctl_list) {
 		ctl_list = list_entry(pos1, struct ctl_list, list);
+		if (ctl_list->ucm_group != ucm_group)
+			continue;
 		if (strcmp(id, snd_ctl_card_info_get_id(ctl_list->ctl_info)) == 0) {
 			card = snd_card_get_index(id);
-			err = uc_mgr_ctl_add(uc_mgr, &ctl_list, ctl, card, info, device, slave);
+			err = uc_mgr_ctl_add(uc_mgr, &ctl_list, ctl, card, info, device + ucm_offset, slave);
 			if (err < 0)
 				goto __nomem;
 			snd_ctl_close(ctl);
+			ctl_list->ucm_group = ucm_group;
 			*ctll = ctl_list;
 			return 0;
 		}
 	}
 
 	ctl_list = NULL;
-	err = uc_mgr_ctl_add(uc_mgr, &ctl_list, ctl, -1, info, device, slave);
+	err = uc_mgr_ctl_add(uc_mgr, &ctl_list, ctl, -1, info, device + ucm_offset, slave);
 	if (err < 0)
 		goto __nomem;
 
+	ctl_list->ucm_group = ucm_group;
 	*ctll = ctl_list;
 	return 0;
 
@@ -320,49 +350,54 @@ const char *uc_mgr_config_dir(int format)
 	return path;
 }
 
-int uc_mgr_config_load(int format, const char *file, snd_config_t **cfg)
+int uc_mgr_config_load_into(int format, const char *file, snd_config_t *top)
 {
 	FILE *fp;
 	snd_input_t *in;
-	snd_config_t *top;
 	const char *default_paths[2];
 	int err;
 
 	fp = fopen(file, "r");
 	if (!fp) {
 		err = -errno;
-  __err0:
+  __err_open:
 		uc_error("could not open configuration file %s", file);
 		return err;
 	}
 	err = snd_input_stdio_attach(&in, fp, 1);
 	if (err < 0)
-		goto __err0;
-	err = snd_config_top(&top);
-	if (err < 0)
-		goto __err1;
+		goto __err_open;
 
 	default_paths[0] = uc_mgr_config_dir(format);
 	default_paths[1] = NULL;
 	err = _snd_config_load_with_include(top, in, 0, default_paths);
 	if (err < 0) {
 		uc_error("could not load configuration file %s", file);
-		goto __err2;
+		if (in)
+			snd_input_close(in);
+		return err;
 	}
 	err = snd_input_close(in);
+	if (err < 0)
+		return err;
+	return 0;
+}
+
+int uc_mgr_config_load(int format, const char *file, snd_config_t **cfg)
+{
+	snd_config_t *top;
+	int err;
+
+	err = snd_config_top(&top);
+	if (err < 0)
+		return err;
+	err = uc_mgr_config_load_into(format, file, top);
 	if (err < 0) {
-		in = NULL;
-		goto __err2;
+		snd_config_delete(top);
+		return err;
 	}
 	*cfg = top;
 	return 0;
-
- __err2:
-	snd_config_delete(top);
- __err1:
-	if (in)
-		snd_input_close(in);
-	return err;
 }
 
 void uc_mgr_free_value(struct list_head *base)
@@ -464,13 +499,22 @@ void uc_mgr_free_sequence_element(struct sequence_element *seq)
 		free(seq->data.cdev);
 		break;
 	case SEQUENCE_ELEMENT_TYPE_CSET:
+	case SEQUENCE_ELEMENT_TYPE_CSET_NEW:
 	case SEQUENCE_ELEMENT_TYPE_CSET_BIN_FILE:
 	case SEQUENCE_ELEMENT_TYPE_CSET_TLV:
+	case SEQUENCE_ELEMENT_TYPE_CTL_REMOVE:
 		free(seq->data.cset);
 		break;
+	case SEQUENCE_ELEMENT_TYPE_SYSSET:
+		free(seq->data.sysw);
+		break;
 	case SEQUENCE_ELEMENT_TYPE_EXEC:
+	case SEQUENCE_ELEMENT_TYPE_SHELL:
 		free(seq->data.exec);
 		break;
+	case SEQUENCE_ELEMENT_TYPE_CFGSAVE:
+		free(seq->data.cfgsave);
+		break;
 	default:
 		break;
 	}
@@ -681,6 +725,7 @@ void uc_mgr_free_verb(snd_use_case_mgr_t *uc_mgr)
 		list_del(&verb->list);
 		free(verb);
 	}
+	uc_mgr_free_sequence(&uc_mgr->fixedboot_list);
 	uc_mgr_free_sequence(&uc_mgr->boot_list);
 	uc_mgr_free_sequence(&uc_mgr->default_list);
 	uc_mgr_free_value(&uc_mgr->value_list);
@@ -698,8 +743,98 @@ void uc_mgr_free_verb(snd_use_case_mgr_t *uc_mgr)
 
 void uc_mgr_free(snd_use_case_mgr_t *uc_mgr)
 {
+	snd_config_delete(uc_mgr->local_config);
 	uc_mgr_free_verb(uc_mgr);
 	uc_mgr_free_ctl_list(uc_mgr);
 	free(uc_mgr->card_name);
 	free(uc_mgr);
 }
+
+/*
+ * UCM card list stuff
+ */
+
+static pthread_mutex_t ucm_cards_mutex = PTHREAD_MUTEX_INITIALIZER;
+static LIST_HEAD(ucm_cards);
+static unsigned int ucm_card_assign;
+
+static snd_use_case_mgr_t *uc_mgr_card_find(unsigned int card_number)
+{
+	struct list_head *pos;
+	snd_use_case_mgr_t *uc_mgr;
+
+	list_for_each(pos, &ucm_cards) {
+		uc_mgr = list_entry(pos, snd_use_case_mgr_t, cards_list);
+		if (uc_mgr->ucm_card_number == card_number)
+			return uc_mgr;
+	}
+	return NULL;
+}
+
+int uc_mgr_card_open(snd_use_case_mgr_t *uc_mgr)
+{
+	unsigned int prev;
+
+	pthread_mutex_lock(&ucm_cards_mutex);
+	prev = ucm_card_assign++;
+	while (uc_mgr_card_find(ucm_card_assign)) {
+		ucm_card_assign++;
+		ucm_card_assign &= 0xffff;
+		/* avoid zero card instance number */
+		if (ucm_card_assign == 0)
+			ucm_card_assign++;
+		if (ucm_card_assign == prev) {
+			pthread_mutex_unlock(&ucm_cards_mutex);
+			return -ENOMEM;
+		}
+	}
+	uc_mgr->ucm_card_number = ucm_card_assign;
+	list_add(&uc_mgr->cards_list, &ucm_cards);
+	pthread_mutex_unlock(&ucm_cards_mutex);
+	return 0;
+}
+
+void uc_mgr_card_close(snd_use_case_mgr_t *uc_mgr)
+{
+	pthread_mutex_lock(&ucm_cards_mutex);
+	list_del(&uc_mgr->cards_list);
+	pthread_mutex_unlock(&ucm_cards_mutex);
+}
+
+/**
+ * \brief Get library configuration based on the private ALSA device name
+ * \param name[in] ALSA device name
+ * \retval config A configuration tree or NULL
+ *
+ * The returned configuration (non-NULL) should be unreferenced using
+ * snd_config_unref() call.
+ */
+const char *uc_mgr_alibcfg_by_device(snd_config_t **top, const char *name)
+{
+	char buf[5];
+	long card_num;
+	snd_config_t *config;
+	snd_use_case_mgr_t *uc_mgr;
+	int err;
+
+	if (strncmp(name, "_ucm", 4) || strlen(name) < 12 || name[8] != '.')
+		return NULL;
+	strncpy(buf, name + 4, 4);
+	buf[4] = '\0';
+	err = safe_strtol_base(buf, &card_num, 16);
+	if (err < 0 || card_num < 0 || card_num > 0xffff)
+		return NULL;
+	config = NULL;
+	pthread_mutex_lock(&ucm_cards_mutex);
+	uc_mgr = uc_mgr_card_find(card_num);
+	/* non-empty configs are accepted only */
+	if (uc_mgr_has_local_config(uc_mgr)) {
+		config = uc_mgr->local_config;
+		snd_config_ref(config);
+	}
+	pthread_mutex_unlock(&ucm_cards_mutex);
+	if (!config)
+		return NULL;
+	*top = config;
+	return name + 9;
+}
